.rej_deletebtn {
    border: 0;
    cursor: pointer;
}
.options-set-container {
    background-color: rgb(239, 239, 239);
    border:1px solid #cfcfcf;
    display:block;
    padding: 10px;
    margin-bottom:10px;
}
