.barcode-container {
    width:100%;
}
.barcode {
    float: right;
}
div.worksheet {
    font-size:0.8em;
}
div.worksheet a {
    color:#000;
    text-decoration:none;
}
div.worksheet h1 {
    padding-top:15px;
    font-size:1.7em;
}
div.subheader {
    padding: 10px 0;
}
div.content {
    padding:20px 0;
    font-size:0.85em;
    border-top:1px solid #333;
}
div.arinfo {
    font-size:0.9em;
    padding: 0 10px;
    font-weight:normal;
}
div.content table {
}
div.content table tbody th {
    padding:0 5px 0 0;
    text-align:left;
}
div.content table tbody td {
    padding:4px;
    border:1px solid #888;
}
div.content table tbody td.requestid {
    font-weight:bold;
}
div.content table tbody td.result {
    width:90px;
}
div.content table tbody th.analyses {
    padding:20px 0 10px 0;
    font-size:1.2em;
}
div.content table td.requestid {
    border:none;
}
div.content table div.barcode {
    float:none;
}
div.content table tbody td.no-result {
    padding:0;
    border:1px solid #fff;
}
div.content table tbody td.no-result div {
    position:relative;
    overflow:hidden;
    padding:30px;
}
div.content table tbody td.no-result div:before {
    content:'';
    display:block;
    position:absolute;
    top:0;
    right:0;
    left:0;
    bottom:0;
    border: 99999px #dcdcdc solid;
    z-index: 0;
}
div.content table tbody td.no-result div * {
    position: relative;
    z-index: 1;
}

@media print {
    div.content table div.barcode {
        margin: 0 0 0 -10px;
    }
}