# -*- coding: utf-8 -*-
#
# This file is part of Bika LIMS
#
# Copyright 2011-2017 by it's authors.
# Some rights reserved. See LICENSE.txt, AUTHORS.txt.

""" Scil Vet abc Plus
"""
# This instrument will use the same parser and importer as Abaxis VetScan VS2, because
# its file results are equal
