.sticker {
    font-family: Helvetica, Arial;
    font-size:6pt;
    /* Total width: 3mm + 38mm + 3mm + 38mm + 3mm = 85mm*/
    width: 85mm;
    height:18mm;
    max-height:18mm;
    max-width:85mm;
    overflow:hidden;
    border-bottom:none;
}
.sticker table.stickers-row {
    width:100%;
    border-spacing: 10px;
    border-collapse: separate;
    background-color:#f5f5f5;
}
.sticker table.stickers-row td.sticker-cell {
    background-color:#fff;
    width:38mm;
    height:18mm;
    max-width:38mm;
    max-height:18mm;
}
.sticker .analysisrequest-info table td.sample-type {
    width:100%;
}
.sticker .analysisrequest-info table,
.sticker .sampling-date-info table {
    border-collapse:collapse;
    font-size: 6pt;
    width:100%;
}
.sticker .analysisrequest-info table td,
.sticker .sampling-date-info table td {
    border:none;
    padding-left:2mm;
}
