# -*- coding: utf-8 -*-
#
# This file is part of Bika LIMS
#
# Copyright 2011-2017 by it's authors.
# Some rights reserved. See LICENSE.txt, AUTHORS.txt.

from add_analyses import AddAnalysesView
from add_blank import AddBlankView
from add_control import AddControlView
from add_duplicate import AddDuplicateView
from add_worksheet import AddWorksheetView
from analyses import AnalysesView
from analyses_transposed import AnalysesTransposedView
from analysisrequests import AnalysisRequestsView
from export import ExportView
from folder import FolderView
from printview import PrintView
from referencesamples import ReferenceSamplesView
from results import ManageResultsView
from services import ServicesView
